<?php

namespace Teknisa\CRM\Service;

use InvalidArgumentException;
use Teknisa\Libs\Util\Environment;

use Teknisa\CRM\Factory\MauticFactory;
use Teknisa\CRM\Util\API\MauticAPI;


class MauticService
{
    protected $mauticFactory;
    protected $mauticAPI;
    protected $newCode;
    protected $environment;
    

    public function __construct(MauticFactory $mauticFactory, MauticAPI $mauticAPI, $newCode, Environment $environment)
    {
        $this->mauticFactory    = $mauticFactory; 
        $this->mauticAPI        = $mauticAPI;
        $this->newCode          = $newCode;
        $this->environment      = $environment;           
    }
    
    public function getParams()
    {
        $dadosApi = $this->mauticFactory->getParams();       
        $this->mauticAPI->conection($dadosApi);
    }

    public function saveParamMautic($row) {

        $nrorg = $this->environment->getNrOrgTrab();
        $cdoperador = $this->environment->getCdOperador();
        $nrorgOper = $this->environment->getNrOrg();
        
        $params = array(
            "DSPUBLICKEYMAUTIC" => $row['DSPUBLICKEYMAUTIC'],
            "DSSECRETKEYMAUTIC" => $row['DSSECRETKEYMAUTIC'],
            "DSBASEURLMAUTIC"   => $row['DSBASEURLMAUTIC'],
            "MAUTICAUTH"        => $row['MAUTICAUTH'],
            "NRORG"             => $nrorg,
            "CDOPERADOR"        => $cdoperador,
            "NRORGOPER"         => $nrorgOper
        );
        
        $this->mauticFactory->saveParamMautic($params);
    }

    public function getLeads()
    {
        $this->getParams();
        return $this->mauticAPI->getContacts();
    }

    public function createContact($params)
    {   
        $this->getParams(); 

        $companyName = $this->mauticFactory->getCompanyName($params['NRPARCNEGOCIO']);  
        
        $createdContact = $this->mauticAPI->createContact($params, $companyName[0]);

        return $createdContact['contact']['id'];
    }

    public function editContact($params)
    {   
        $this->getParams(); 

        $companyName = $this->mauticFactory->getCompanyName($params['NRPARCNEGOCIO']);  
        $this->mauticAPI->editContact($params, $companyName[0]);
    }

    public function novoInterlocutor($params)
    {
        $this->getParams(); 

        foreach ($params as $contact) {
            $novoContato = $contact['contact'] ?? null;

            if($novoContato){
                $id         = $novoContato["id"];
                $name       = $novoContato['fields']['core']['firstname']['value'] ?? '';
                $company    = $novoContato['fields']['core']['company']['value'] ?? '';
                $email      = $novoContato['fields']['core']['email']['value'] ?? '';
                $mobile     = $novoContato['fields']['core']['mobile']['value'] ?? '';

                if($company && !empty($company)){
                    $companySearch = $this->searchParc([
                        'NMPRINCIPALPARC'=> $company,
                        'NRORG'          => 20
                    ]); 
                }
                
                
                $dataToInsert = [
                    'IDMAUTIC'       => $id,
                    'NMINTERLOCUTOR' => $name,
                    'EMAILINTERLOC'  => $email, 
                    'NRTELEFONE1'    => $mobile,
                    'NMPRINCIPALPARC'=> $company,
                    'NRSEQINTERLOCUTOR'  => $this->newCode->getCodeByTableName("CRM_INTERLOCUTOR", 20),
                    'NRORG'          => 20,
                ];   
                
                $this->mauticFactory->insertInterlocutor($dataToInsert);
            }

        }
    }

    public function saveProspect($params)
    {
        $this->getParams(); 

        foreach ($params as $company){
            $companyData = $company['company'] ?? null;

            if($companyData && !empty($companyData['PastChanges'])){
                
                $changes = $companyData['PastChanges']['fields'];

                $companyNameSearch = isset($changes['companyname']) ? $changes['companyname'][0] : $companyData['name'];
                
                $companySearch = $this->searchParc([
                    'NMPRINCIPALPARC'=> $companyNameSearch,
                    'NRORG'          => 20
                ]); 
                                
                if((isset($changes['companyname']) && !($changes['companyname'][0]) && empty($companySearch)) || empty($companySearch) )
                {
                    $nrparceiro     = $this->newCode->getCodeByTableName("PARCNEGOCIO", 20);
                    $nrcandidato    = $this->newCode->getCodeByTableName("CANDIDATO9999", 20);

                    $insertParc = [
                        'IDINSTITUICAO'         => 'S',
                        'IDPARCFUNDIDO'         => 'N',
                        'CDTIPOINSCRICAO'       => 'LIVRE',
                        "CDTIPOPARCPRINCIPAL"   => 'CANDIDATO',
                        'NRPARCNEGOCIO'         => $nrparceiro,
                        'NRORG'                 => 20,
                        'NMPRINCIPALPARC'       => $companyData['name'],
                        'NMSECUNDARIPARC'       => $companyData['name'],
                        'IDATIVO'               => 'S',
                        'NRORGINCLUSAO'         => 20,
                    ];
                    $this->mauticFactory->novoParcnegocio($insertParc);
                    
                    $insertLead = [
                        'IDATIVO'           => 'S',
                        'IDSTATUSCAN'       => '2',
                        'IDPARCFUNDIDO'     => 'N',  
                        'NMCANDIDATO'       => $companyData['name'],
                        'NMFANTCAN'         => $companyData['name'],
                        'NRCANDIDATO'       => $nrcandidato,
                        'NRPARCNEGOCIO'     => $nrparceiro,
                        'NRORG'             => 20,
                        'IDOPERATIVO'       => 'A',
                        "IDTPIJURCAN"       => 'I',
                        "IDSTATUSCAN"       => '2',
                        "NRORGINCLUSAO"     => 20,
                    ];
                    $this->mauticFactory->insertLead($insertLead);

                    $insertStatus = [
                        'NRPARCNEGOCIO'     => $nrparceiro,
                        'STATUSPARC'        => 'CANDIDATO',
                        'NRORG'             => 20,
                        'CRM_NRORGTRAB'     => 20,
                        'IDATIVO'           => 'S',
                    ];
                    $this->mauticFactory->insertStatusParc($insertStatus);                  

                } else {                    
                    // em caso de ediçao
                    $updatedParcnegocio = [];

                    foreach ($changes as $field => $change) {
                        $mappedField = $this->mapFieldName($field);

                        if ($mappedField) { // Apenas campos mapeados
                            $updatedParcnegocio[$mappedField] = $change[1];
                        }
                    }

                    // Atualizar parceiro (PARCNEGOCIO)
                    if (!empty($updatedParcnegocio)) {
                        $dataToUpdate = array_merge([
                            'IDATIVO'        => 'S',
                        ], $updatedParcnegocio);

                        $params = [
                            'NRPARCNEGOCIO'  => $companySearch[0]['NRPARCNEGOCIO'],
                            'NRORG'          => 20,
                        ];

                        $this->mauticFactory->updateParcnegocio($dataToUpdate, $params);
                    }

                }
            }
        }
    }
    
    private function mapFieldName($field)
    {
        $fieldMap = [
            'companyname'    => 'NMPRINCIPALPARC',           
        ];

        return $fieldMap[$field] ?? null;
    }
    
    public function searchParc($companyData){
        return $this->mauticFactory->verifyProspectName($companyData);         
    }



    public function createCompany($nome)
    {
        $this->getParams();
        
        $this->mauticAPI->createCompany($nome);
    }


    
}
